@@ -1,7 +1,7 @@
 /*
-    $Id: main.c,v 1.18 2004/10/08 21:05:04 airborne Exp $
+    $Id: main.c,v 1.27 2005/07/23 09:42:16 airborne Exp $
 
-    Copyright (C) 2003, 2004 Kris Verbeeck <airborne@advalvas.be>
+    Copyright (C) 2003, 2004, 2005 Kris Verbeeck <airborne@advalvas.be>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -40,10 +40,7 @@
 #define HTTP_PREFIX_LEN 7
 
 /* parsed command-line parameters */
-#define CMD_NONE   0
-#define CMD_DISCID 1
-#define CMD_QUERY  2
-#define CMD_READ   3
+enum { CMD_NONE = 0, CMD_DISCID, CMD_QUERY, CMD_READ, CMD_SITES, CMD_SEARCH };
 static int quiet = 0;           /* work silently, reports no errors */
 static int command = 0;         /* request command */
 static char *category = NULL;   /* category command-line argument */
@@ -56,6 +53,7 @@ static char *device = NULL;     /* device to use if use_cd == 1. NULL means
                                    to find a suitable CD-ROM drive. */
 static int use_time = 0;        /* use track times (in seconds) instead of frame offsets */
 static char *charset = NULL;    /* requested character set encoding */
+static char *searchstr = NULL;  /* text search string */
 
 /* print usage message */
 static void usage(void)
@@ -84,6 +82,8 @@ static void usage(void)
     fprintf(stderr, "  query <len> <n> <fo_1> ... <fo_n>\n");
     fprintf(stderr, "                   query CDDB server and list all matching entries\n");
     fprintf(stderr, "  read <cat> <id>  retrieve disc details from CDDB server\n");
+    fprintf(stderr, "  search <str>     perform a text search against the CDDB database\n");
+    fprintf(stderr, "  sites            retrieve a list of mirror sites\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Command arguments\n");
     fprintf(stderr, "  <cat>            disc category (see below)\n");
@@ -92,6 +92,7 @@ static void usage(void)
     fprintf(stderr, "  <id>             disc ID in hexadecimal\n");
     fprintf(stderr, "  <len>            disc length in seconds\n");
     fprintf(stderr, "  <n>              track count\n");
+    fprintf(stderr, "  <str>            search string\n");
     fprintf(stderr, "\n");
 #ifdef HAVE_LIBCDIO
     fprintf(stderr, "If you do not specify any arguments for a command, the program\n");
@@ -117,6 +118,10 @@ static void usage(void)
     fprintf(stderr, "  To read the details of the CD 'Mezzanine' from Massive Attack:\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "\tcddb_query read misc 0x920ef00b\n");
+    fprintf(stderr, "\n");
+    fprintf(stderr, "  To search for all CDs with the string 'Mezzanine':\n");
+    fprintf(stderr, "\n");
+    fprintf(stderr, "\tcddb_query search Mezzanine\n");
 }
 
 /* print error message */
@@ -181,7 +186,7 @@ static void parse_disc_data(int cmd, int argc, char **argv, int idx)
 static void init_protocol(cddb_conn_t *conn, const char *proto)
 {
     int len, port;
-    char *aux, *host, *portstr;
+    char *aux, *host, *portstr, *password;
 
     if (!*optarg) {
         error_usage("-P, server protocol missing");
@@ -214,11 +219,22 @@ static void init_protocol(cddb_conn_t *conn, const char *proto)
             error_exit(GENERIC_ERROR, "environment variable 'http_proxy' invalid");
         }
         host = aux + HTTP_PREFIX_LEN;
+        /* Check if a proxy username:password pair is provided */
+        aux = strchr(host, '@');
+        if (aux != NULL) {
+            *aux = '\0';
+            password = strchr(host, ':');
+            if (password != NULL) {
+                *password = '\0';
+                password++;
+            }
+            cddb_set_http_proxy_credentials(conn, host, password);
+            host = aux + 1;
+        }
         /* Check if a proxy port is specified. */
         portstr = strchr(host, ':');
         if (portstr == NULL) {
-            /* No port, set proxy server name and use default port
-               80. */
+            /* No port, set proxy server name and use default port 80. */
             cddb_set_http_proxy_server_name(conn, host);
             cddb_set_http_proxy_server_port(conn, 80);
         } else {
@@ -237,7 +253,6 @@ static void init_protocol(cddb_conn_t *conn, const char *proto)
             cddb_set_http_proxy_server_port(conn, port);
         }
     } else {
-        /* XXX: get proxy settings from env var 'http_proxy'!! */
         error_usage("-P, invalid server protocol '%s'", optarg);
     }
 }
@@ -392,6 +407,23 @@ static void parse_cmdline(int argc, char **argv, cddb_conn_t *conn)
                 error_usage("the read command requires two arguments");
             }
         }
+    } else if (strcmp(argv[optind], "sites") == 0) {
+        /* CDDB sites */
+        command = CMD_SITES;
+        if (argc != optind + 1) {
+            error_usage("the sites command expects no arguments");
+        }
+        use_cd = 0;
+    } else if (strcmp(argv[optind], "search") == 0) {
+        /* CDDB search */
+        command = CMD_SEARCH;
+        if (argc == optind + 2) {
+            /* one more argument is needed */
+            searchstr = strdup(argv[optind+1]);
+        } else {
+            error_usage("the search command requires one argument");
+        }
+        use_cd = 0;
     } else {
         /* unknown command */
         error_usage("unknown command '%s'", argv[optind]);
@@ -443,6 +475,13 @@ int main(int argc, char **argv)
         /* The frame offset data is no longer needed because it is now
            also present in the disc structure. */
         FREE_NOT_NULL(foffset);
+    } else if (command == CMD_SEARCH) {
+        /* Create an empty disc structure for the search command.  It
+           will be used to store the search results. */
+        disc = cddb_disc_new();
+        if (!disc) {
+            error_exit(GENERIC_ERROR, "could not create disc structure");
+        }
     }
 
     /* Execute requested command. */
@@ -491,6 +530,14 @@ int main(int argc, char **argv)
         /* Let's display the information that was read. */
         do_display(disc);
         break;
+    case CMD_SITES:
+        /* Display information about mirror sites. */
+        do_sites(conn);
+        break;
+    case CMD_SEARCH:
+        /* Search the CDDB database for possibly matches. */
+        do_search(conn, disc, searchstr, quiet);
+        break;
     }
     /* Finally, we have to clean up.  With the cddb_disc_destroy
        function we can easily free all memory used by a single disc.
@@ -502,5 +549,13 @@ int main(int argc, char **argv)
     cddb_disc_destroy(disc);
     cddb_destroy(conn);
 
+    /* Also destroy any global resources reserved by the library. */
+    libcddb_shutdown();
+
+    FREE_NOT_NULL(charset);
+    FREE_NOT_NULL(device);
+    FREE_NOT_NULL(category);
+    FREE_NOT_NULL(searchstr);
+
     return 0;
 }
