@@ -1,7 +1,7 @@
 /*
-    $Id: cddb.h,v 1.9 2004/07/18 07:23:09 airborne Exp $
+    $Id: cddb.h,v 1.12 2005/05/29 08:06:11 airborne Exp $
 
-    Copyright (C) 2003, 2004 Kris Verbeeck <airborne@advalvas.be>
+    Copyright (C) 2003, 2004, 2005 Kris Verbeeck <airborne@advalvas.be>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,6 +22,8 @@
 #ifndef CDDB_H
 #define CDDB_H 1
 
+#include <cddb/version.h>
+
 #ifdef __cplusplus
     extern "C" {
 #endif
@@ -31,6 +33,7 @@
 #include <cddb/cddb_error.h>
 #include <cddb/cddb_track.h>
 #include <cddb/cddb_disc.h>
+#include <cddb/cddb_site.h>
 #include <cddb/cddb_conn.h>
 #include <cddb/cddb_cmd.h>
 #include <cddb/cddb_log.h>
@@ -41,6 +44,22 @@
  */
 
 
+/**
+ * Initializes the library.  This is used to setup any globally used
+ * variables.  The first time you create a new CDDB connection structure
+ * the library will automatically initialize itself.  So, there is no
+ * need to explicitly call this function.
+ */
+void libcddb_init(void);
+
+/**
+ * Frees up any global (cross connection) resources.  You should call
+ * this function before terminating your program.  Using any library
+ * calls after shutting down are bound to give problems.
+ */
+void libcddb_shutdown(void);
+
+
 #ifdef __cplusplus
     }
 #endif
