@@ -1,7 +1,7 @@
 /*
-    $Id: cddb_conn.h,v 1.22 2004/10/15 18:40:08 airborne Exp $
+    $Id: cddb_conn.h,v 1.30 2005/08/03 18:25:01 airborne Exp $
 
-    Copyright (C) 2003, 2004 Kris Verbeeck <airborne@advalvas.be>
+    Copyright (C) 2003, 2004, 2005 Kris Verbeeck <airborne@advalvas.be>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -30,11 +30,16 @@
 #include <stdio.h>
 #include <netinet/in.h>
 
-#define CACHE_OFF  0            /**< do not use local CDDB cache, network
+#include "cddb/cddb_site.h"
+
+
+typedef enum {
+    CACHE_OFF = 0,              /**< do not use local CDDB cache, network
                                      only */
-#define CACHE_ON   1            /**< use local CDDB cache, if possible */
-#define CACHE_ONLY 2            /**< only use local CDDB cache, no network
+    CACHE_ON,                   /**< use local CDDB cache, if possible */
+    CACHE_ONLY                  /**< only use local CDDB cache, no network
                                      access */
+} cddb_cache_mode_t;
 
 /**
  * Forward declaration of opaque structure used for character set
@@ -46,66 +51,26 @@ typedef struct cddb_iconv_s *cddb_iconv_t;
  * An opaque structure for keeping state about the connection to a
  * CDDB server.
  */
-typedef struct cddb_conn_s 
-{
-    unsigned int buf_size;      /**< maximum line/buffer size, defaults to 1024
-                                     (see DEFAULT_BUF_SIZE) */
-    char *line;                 /**< last line read */
+typedef struct cddb_conn_s cddb_conn_t;
 
-    int is_connected;           /**< are we already connected to the server? */
-    struct sockaddr_in sa;      /**< the socket address structure for
-                                     connecting to the CDDB server */
-    int socket;                 /**< the socket file descriptor */
-    char *server_name;          /**< host name of the CDDB server, defaults
-                                     to 'freedb.org' (see DEFAULT_SERVER) */
-    int server_port;            /**< port of the CDDB server, defaults to 888 
-                                     (see DEFAULT_PORT) */
-    int timeout;                /**< time out interval (in seconds) used during
-                                     network operations, defaults to 10 seconds
-                                     (see DEFAULT_TIMEOUT) */
+/**
+ * Which fields to use for the full text search is defined by one or
+ * more of the constants below.
+ */
+typedef enum {
+    SEARCH_NONE = 0,            /**< no fields */
+    SEARCH_ARTIST = 1,          /**< artist name field */
+    SEARCH_TITLE = 2,           /**< disc title field */
+    SEARCH_TRACK = 4,           /**< track title field */
+    SEARCH_OTHER = 8,           /**< other fields */
+    SEARCH_ALL = ~0,            /**< all fields */
+} cddb_search_t;
 
-    char *http_path_query;      /**< URL for querying the server through HTTP,
-                                     defaults to /~cddb/cddb.cgi'
-                                     (see DEFAULT_PATH_QUERY) */
-    char *http_path_submit;     /**< URL for submitting to the server through HTTP,
-                                     defaults to /~cddb/submit.cgi'
-                                     (see DEFAULT_PATH_SUBMIT) */
-    int is_http_enabled;        /**< use HTTP, disabled by default */
-
-    int is_http_proxy_enabled;  /**< use HTTP through a proxy server,
-                                     disabled by default */
-    char *http_proxy_server;    /**< host name of the HTTP proxy server */
-    int http_proxy_server_port; /**< port of the HTTP proxy server,
-                                     defaults to 8080 (see DEFAULT_PROXY_PORT) */
-
-    FILE *cache_fp;             /**< a file pointer to a cached CDDB entry or
-                                     NULL if no cached version is available */
-    int use_cache;              /**< field to specify local CDDB cache behaviour, 
-                                     enabled by default (CACHE_ON) */
-    char *cache_dir;            /**< CDDB slave cache, defaults to 
-                                     '~/.cddbslave' (see DEFAULT_CACHE) */
-    int cache_read;             /**< read data from cached file instead of
-                                     from the network */
-
-    char *cname;                /**< name of the client program, 'libcddb' by
-                                     default */
-    char *cversion;             /**< version of the client program, current 
-                                     libcddb version by default */
-    char *user;                 /**< user name supplied to CDDB server, defaults
-                                     to the value of the 'USER' environment 
-                                     variable or 'anonymous' if undefined */
-    char *hostname;             /**< host name of the local machine, defaults
-                                     to the value of the 'HOSTNAME' environment
-                                     variable or 'localhost' if undefined */
-
-    cddb_error_t errnum;        /**< error number of last CDDB command */
-
-    cddb_disc_t **query_data;   /**< data structure to keep CDDB query results */
-    int query_idx;              /**< iterator index for query result set */
-    int query_cnt;              /**< number of entries in query result set */
-
-    cddb_iconv_t charset;       /**< character set conversion settings */
-} cddb_conn_t;
+/**
+ * Macro to be used for building the category search bit-string from
+ * the values of #cddb_cat_t.
+ */
+#define SEARCHCAT(c) (1 << (c))
 
 
 /* --- construction / destruction --- */
@@ -138,9 +103,9 @@ void cddb_destroy(cddb_conn_t *c);
  * converted.
  *
  * @param c The connection structure.
- * @param charset The character set that will be used.
- * @return FALSE if the specified character set is unknown, or no conversion
- *         from/to UTF-8 is available.  TRUE otherwise.
+ * @param cs The character set that will be used.
+ * @return False if the specified character set is unknown, or no conversion
+ *         from/to UTF-8 is available.  True otherwise.
  */
 int cddb_set_charset(cddb_conn_t *c, const char *cs);
 
@@ -152,6 +117,21 @@ int cddb_set_charset(cddb_conn_t *c, const char *cs);
  */
 void cddb_set_buf_size(cddb_conn_t *c, unsigned int size);
 
+/**
+ * Set all server details in one go through the use of a site structure.  This
+ * function initializzes the server address, port, protocol and query path in
+ * case of HTTP.
+ *
+ * @see cddb_sites
+ * @see cddb_first_site
+ * @see cddb_next_site
+ *
+ * @param c The connection structure.
+ * @param site The site to use.
+ * @return Error code: CDDB_ERR_OK or CDDB_ERR_INVALID.
+ */
+cddb_error_t cddb_set_site(cddb_conn_t *c, const cddb_site_t *site);
+
 /**
  * Get the host name of the CDDB server that is currently being used.
  *
@@ -160,7 +140,7 @@ void cddb_set_buf_size(cddb_conn_t *c, unsigned int size);
  * @param c The connection structure.
  * @return The server host name.
  */
-#define cddb_get_server_name(c) (c)->server_name
+const char *cddb_get_server_name(const cddb_conn_t *c);
 
 /**
  * Set the host name of the CDDB server.  The default value for the
@@ -181,7 +161,7 @@ void cddb_set_server_name(cddb_conn_t *c, const char *server);
  * @param c The connection structure.
  * @return The server port.
  */
-#define cddb_get_server_port(c) (c)->server_port
+unsigned int cddb_get_server_port(const cddb_conn_t *c);
 
 /**
  * Set the port of the CDDB server.  The default value is 888.
@@ -201,7 +181,7 @@ void cddb_set_server_port(cddb_conn_t *c, int port);
  * @param c The connection structure.
  * @return The current time out in seconds.
  */
-#define cddb_get_timeout(c) (c)->timeout
+unsigned int cddb_get_timeout(const cddb_conn_t *c);
 
 /**
  * Set the network time out value (in seconds).  The default is 10
@@ -212,7 +192,7 @@ void cddb_set_server_port(cddb_conn_t *c, int port);
  * @param c The connection structure.
  * @param t The new time out in seconds.
  */
-#define cddb_set_timeout(c, t) (c)->timeout = t
+void cddb_set_timeout(cddb_conn_t *c, unsigned int t);
 
 /**
  * Get the URL path for querying a CDDB server through HTTP.
@@ -222,7 +202,7 @@ void cddb_set_server_port(cddb_conn_t *c, int port);
  * @param c The connection structure.
  * @return The URL path.
  */
-#define cddb_get_http_path_query(c) (c)->http_path_query
+const char *cddb_get_http_path_query(const cddb_conn_t *c);
 
 /**
  * Set the URL path for querying a CDDB server through HTTP.  The
@@ -243,7 +223,7 @@ void cddb_set_http_path_query(cddb_conn_t *c, const char *path);
  * @param c The connection structure.
  * @return The URL path.
  */
-#define cddb_get_http_path_submit(c) (c)->http_path_submit
+const char *cddb_get_http_path_submit(const cddb_conn_t *c);
 
 /**
  * Set the URL path for submitting to a CDDB server through HTTP.  The
@@ -266,7 +246,7 @@ void cddb_set_http_path_submit(cddb_conn_t *c, const char *path);
  * @param c The CDDB connection structure.
  * @return True or false.
  */
-#define cddb_is_http_enabled(c) (c)->is_http_enabled
+unsigned int cddb_is_http_enabled(const cddb_conn_t *c);
 
 /**
  * Enable HTTP tunneling to connect to the CDDB server.  By default
@@ -302,7 +282,7 @@ void cddb_http_disable(cddb_conn_t *c);
  * @param c The CDDB connection structure.
  * @return True or false.
  */
-#define cddb_is_http_proxy_enabled(c) (c)->is_http_proxy_enabled
+unsigned int cddb_is_http_proxy_enabled(const cddb_conn_t *c);
 
 /**
  * Enable HTTP tunneling through an HTTP proxy server to connect to
@@ -336,7 +316,7 @@ void cddb_http_proxy_disable(cddb_conn_t *c);
  * @param c The connection structure.
  * @return The proxy server host name.
  */
-#define cddb_get_http_proxy_server_name(c) (c)->http_proxy_server
+const char *cddb_get_http_proxy_server_name(const cddb_conn_t *c);
 
 /**
  * Set the host name of the HTTP proxy server.  There is no default
@@ -357,7 +337,7 @@ void cddb_set_http_proxy_server_name(cddb_conn_t *c, const char *server);
  * @param c The connection structure.
  * @return The proxy server port.
  */
-#define cddb_get_http_proxy_server_port(c) (c)->http_proxy_server_port
+unsigned int cddb_get_http_proxy_server_port(const cddb_conn_t *c);
 
 /**
  * Set the port of the HTTP proxy server.  The default value is 8080.
@@ -369,13 +349,61 @@ void cddb_set_http_proxy_server_name(cddb_conn_t *c, const char *server);
  */
 void cddb_set_http_proxy_server_port(cddb_conn_t *c, int port);
 
+/**
+ * Set the HTTP proxy user name which is used when Basic Authentication
+ * is required.
+ *
+ * @param c        The connection structure.
+ * @param username The user name.
+ */
+void cddb_set_http_proxy_username(cddb_conn_t* c, const char* username);
+
+/**
+ * Get the HTTP proxy user name.
+ *
+ * @param c The connection structure.
+ * @return The user name.
+ */
+const char *cddb_get_http_proxy_username(const cddb_conn_t *c);
+
+/**
+ * Set the HTTP proxy password which is used when Basic Authentication
+ * is required.
+ *
+ * @param c      The connection structure.
+ * @param passwd The password.
+ */
+void cddb_set_http_proxy_password(cddb_conn_t* c, const char* passwd);
+
+/**
+ * Get the HTTP proxy password. 
+ *
+ * @param c The connection structure.
+ * @return The password.
+ */
+const char *cddb_get_http_proxy_password(const cddb_conn_t *c);
+
+/**
+ * Set the HTTP proxy user name and password in one go.  These
+ * credentials are used when Basic Authentication is required.  The
+ * advantage of using this function over setting the user name and
+ * password seperately is that the cleartext user name and password
+ * are not kept in memory longer than needed.
+ *
+ * @param c        The connection structure.
+ * @param username The user name.
+ * @param passwd   The password.
+ */
+void cddb_set_http_proxy_credentials(cddb_conn_t* c,
+                                     const char *username, const char* passwd);
+
 /**
  * Get the error number returned by the last libcddb command.
  *
  * @param c The CDDB connection structure.
  * @return The error number.
  */
-#define cddb_errno(c) (c)->errnum
+cddb_error_t cddb_errno(const cddb_conn_t *c);
 
 /**
  * Set the name and version of the client program overwriting the
@@ -414,7 +442,7 @@ int cddb_set_email_address(cddb_conn_t *c, const char *email);
  *
  * @param c The connection structure.
  */
-#define cddb_cache_mode(c) (c)->use_cache
+cddb_cache_mode_t cddb_cache_mode(const cddb_conn_t *c);
 
 /**
  * Enable caching of CDDB entries locally.  Caching is enabled by
@@ -427,7 +455,7 @@ int cddb_set_email_address(cddb_conn_t *c, const char *email);
  *
  * @param c The connection structure.
  */
-#define cddb_cache_enable(c) (c)->use_cache = CACHE_ON
+void cddb_cache_enable(cddb_conn_t *c);
 
 /**
  * Only use the local CDDB cache.  Never contact a server to retrieve
@@ -440,7 +468,7 @@ int cddb_set_email_address(cddb_conn_t *c, const char *email);
  *
  * @param c The connection structure.
  */
-#define cddb_cache_only(c) (c)->use_cache = CACHE_ONLY
+void cddb_cache_only(cddb_conn_t *c);
 
 /**
  * Disable caching of CDDB entries locally.  All data will be fetched
@@ -453,7 +481,7 @@ int cddb_set_email_address(cddb_conn_t *c, const char *email);
  *
  * @param c The connection structure.
  */
-#define cddb_cache_disable(c) (c)->use_cache = CACHE_OFF
+void cddb_cache_disable(cddb_conn_t *c);
 
 /**
  * Return the directory currently being used for caching.
@@ -463,7 +491,7 @@ int cddb_set_email_address(cddb_conn_t *c, const char *email);
  * @param c The connection structure.
  * @return The directory being used for caching.
  */
-#define cddb_cache_get_dir(c) (c)->cache_dir;
+const char *cddb_cache_get_dir(const cddb_conn_t *c);
 
 /**
  * Change the directory used for caching CDDB entries locally.  The
@@ -479,6 +507,51 @@ int cddb_set_email_address(cddb_conn_t *c, const char *email);
  */
 int cddb_cache_set_dir(cddb_conn_t *c, const char *dir);
 
+/**
+ * Retrieve the first CDDB mirror site.
+ *
+ * @param c The connection structure.
+ * @return The first mirror site or NULL if not found.
+ */
+const cddb_site_t *cddb_first_site(cddb_conn_t *c);
+
+/**
+ * Retrieve the next CDDB mirror site.
+ *
+ * @param c The connection structure.
+ * @return The next mirror site or NULL if not found.
+ */
+const cddb_site_t *cddb_next_site(cddb_conn_t *c);
+
+/**
+ * Set the bit-string specifying which fields to examine when
+ * performing a text search.  By default only the artist and disc
+ * title fields are searched.
+ *
+ * @param c The connection structure.
+ * @param fields A bitwise ORed set of values from #cddb_search_t.
+ */
+void cddb_search_set_fields(cddb_conn_t *c, unsigned int fields);
+
+/**
+ * Set the bit-string specifying which categories to examine when
+ * performing a text search.  The #SEARCHCAT macro needs to be used to
+ * build the actual bit-string from individual categories.  The
+ * #cddb_search_t values #SEARCH_NONE and #SEARCH_ALL are also valid.
+ * The example below shows some possible combinations.  By default all
+ * categories are searched.
+ *
+ * @code
+ * unsigned int cats = SEARCHCAT(CDDB_CAT_ROCK) | SEARCHCAT(CDDB_CAT_MISC);
+ * unsigned int cats = SEARCH_ALL;
+ * unsigned int cats = SEARCH_NONE;
+ * @endcode
+ *
+ * @param c The connection structure.
+ * @param cats A bitwise ORed set of values from #SEARCHCAT(#cddb_cat_t).
+ */
+void cddb_search_set_categories(cddb_conn_t *c, unsigned int cats);
+
 
 #ifdef __cplusplus
     }
