@@ -1,7 +1,7 @@
 /*
-    $Id: cddb_cmd.c,v 1.51 2004/10/16 06:32:52 airborne Exp $
+    $Id: cddb_cmd.c,v 1.61 2005/08/04 18:44:27 airborne Exp $
 
-    Copyright (C) 2003, 2004 Kris Verbeeck <airborne@advalvas.be>
+    Copyright (C) 2003, 2004, 2005 Kris Verbeeck <airborne@advalvas.be>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -28,6 +28,7 @@
 #include <unistd.h>
 
 #include "cddb/cddb_ni.h"
+#include "cddb/ll.h"
 
 
 static const char *CDDB_COMMANDS[CMD_LAST] = {
@@ -37,10 +38,12 @@ static const char *CDDB_COMMANDS[CMD_LAST] = {
     "cddb query %08x %d %s %d",
     "cddb write %s %08x",
     "proto %d",
+    "sites",
+    /* special full text search command (only HTTP) */
+    "words=%s%s",
 };
 
 #define WRITE_BUF_SIZE 4096
-#define QUERY_RESULT_SET_INC 10
 
 
 /*
@@ -66,52 +69,34 @@ char *cddb_read_line(cddb_conn_t *c);
  */
 int cddb_write_data(cddb_conn_t *c, char *buf, int size, cddb_disc_t *disc);
 
-/**
- */
 int cddb_http_parse_response(cddb_conn_t *c);
 
-/**
- */
 void cddb_http_parse_headers(cddb_conn_t *c);
 
-/**
- */
-int cddb_http_send_cmd(cddb_conn_t *c, int cmd, va_list args);
+int cddb_http_send_cmd(cddb_conn_t *c, cddb_cmd_t cmd, va_list args);
 
-/**
- */
 int cddb_parse_record(cddb_conn_t *c, cddb_disc_t *disc);
 
-/**
- */
 int cddb_parse_query_data(cddb_conn_t *c, cddb_disc_t *disc, const char *line);
 
-/**
- */
+static int cddb_parse_search_data(cddb_conn_t *c, cddb_disc_t **disc,
+                                  char *line, regmatch_t *matches);
+
+static void cddb_search_param_str(cddb_search_params_t *params,
+                                  char *buf, int len);
+
 char *cddb_cache_file_name(cddb_conn_t *c, cddb_disc_t *disc);
 
-/**
- */
 int cddb_cache_exists(cddb_conn_t *c, cddb_disc_t *disc);
 
-/**
- */
 int cddb_cache_open(cddb_conn_t *c, cddb_disc_t *disc, const char* mode);
 
-/**
- */
 void cddb_cache_close(cddb_conn_t *c);
 
-/**
- */
 int cddb_cache_read(cddb_conn_t *c, cddb_disc_t *disc);
 
-/**
- */
 int cddb_cache_query(cddb_conn_t *c, cddb_disc_t *disc);
 
-/**
- */
 int cddb_cache_query_disc(cddb_conn_t *c, cddb_disc_t *disc);
 
 /**
@@ -119,8 +104,6 @@ int cddb_cache_query_disc(cddb_conn_t *c, cddb_disc_t *disc);
  */
 void cddb_cache_query_init(void);
 
-/**
- */
 int cddb_cache_mkdir(cddb_conn_t *c, cddb_disc_t *disc);
 
 
@@ -332,26 +315,6 @@ int cddb_cache_mkdir(cddb_conn_t *c, cddb_disc_t *disc)
 }
 
 
-/* --- miscellaneous --- */
-
-
-void cddb_query_clear(cddb_conn_t *c)
-{
-    int i;
-
-    cddb_log_debug("cddb_query_clear()");
-    if (c->query_data != NULL) {
-        for (i = 0; i < c->query_cnt; i++) {
-            cddb_disc_destroy(c->query_data[i]);
-        }
-        free(c->query_data);
-        c->query_data = NULL;
-        c->query_idx = 0;
-        c->query_cnt = 0;
-    }
-}
-
-
 /* --- server request / response handling --- */
 
 
@@ -448,6 +411,10 @@ int cddb_http_parse_response(cddb_conn_t *c)
         case 200:
             /* HTTP OK */
             break;
+        case 407:
+            cddb_errno_log_error(c, CDDB_ERR_PROXY_AUTH);
+            return FALSE;
+            break;
         default:
             /* anythign else = error */
             cddb_errno_log_error(c, CDDB_ERR_SERVER_ERROR);
@@ -469,7 +436,16 @@ void cddb_http_parse_headers(cddb_conn_t *c)
     }
 }
 
-int cddb_http_send_cmd(cddb_conn_t *c, int cmd, va_list args)
+static int cddb_add_proxy_auth(cddb_conn_t *c)
+{
+    /* send proxy authorization if credentials are set */
+    if (c->http_proxy_auth) {
+        sock_fprintf(c, "Proxy-Authorization: Basic %s\r\n", c->http_proxy_auth);
+    }
+    return TRUE;
+}
+
+int cddb_http_send_cmd(cddb_conn_t *c, cddb_cmd_t cmd, va_list args)
 {
     cddb_log_debug("cddb_http_send_cmd()");
     switch (cmd) {
@@ -489,6 +465,7 @@ int cddb_http_send_cmd(cddb_conn_t *c, int cmd, va_list args)
                                  c->server_name, c->server_port, c->http_path_submit);
                     sock_fprintf(c, "Host: %s:%d\r\n",
                                  c->server_name, c->server_port);
+                    cddb_add_proxy_auth(c);
                 } else {
                     /* direct connection */
                     sock_fprintf(c, "POST %s HTTP/1.0\r\n", c->http_path_submit);
@@ -510,11 +487,11 @@ int cddb_http_send_cmd(cddb_conn_t *c, int cmd, va_list args)
                 
                 if (c->is_http_proxy_enabled) {
                     /* use an HTTP proxy */
-                    sock_fprintf(c, "GET http://%s:%d%s", 
+                    sock_fprintf(c, "GET http://%s:%d%s?", 
                                  c->server_name, c->server_port, c->http_path_query);
                 } else {
                     /* direct connection */
-                    sock_fprintf(c, "GET %s", c->http_path_query);
+                    sock_fprintf(c, "GET %s?", c->http_path_query);
                 }
 
                 buf = (char*)malloc(c->buf_size);
@@ -525,18 +502,22 @@ int cddb_http_send_cmd(cddb_conn_t *c, int cmd, va_list args)
                     return FALSE;
                 }
                 url_encode(buf);
-                sock_fprintf(c, "?cmd=%s&", buf);
+                if (cmd == CMD_SEARCH) {
+                    sock_fprintf(c, "%s", buf);
+                } else {
+                    sock_fprintf(c, "cmd=%s&", buf);
+                    sock_fprintf(c, "hello=%s+%s+%s+%s&", 
+                                 c->user, c->hostname, c->cname, c->cversion);
+                    sock_fprintf(c, "proto=%d", DEFAULT_PROTOCOL_VERSION);
+                }
                 free(buf);
-
-                sock_fprintf(c, "hello=%s+%s+%s+%s&", 
-                             c->user, c->hostname, c->cname, c->cversion);
-                sock_fprintf(c, "proto=%d", DEFAULT_PROTOCOL_VERSION);
                 sock_fprintf(c, " HTTP/1.0\r\n");
 
                 if (c->is_http_proxy_enabled) {
                     /* insert host header */
                     sock_fprintf(c, "Host: %s:%d\r\n",
                                  c->server_name, c->server_port);
+                    cddb_add_proxy_auth(c);
                 }
                 sock_fprintf(c, "\r\n");
 
@@ -659,11 +640,11 @@ int cddb_parse_record(cddb_conn_t *c, cddb_disc_t *disc)
                     }
                     track->frame_offset = cddb_regex_get_int(line, matches, 1);
                     track_no++;
+                    break;
                 } else {
                     /* expect disc length now */
                     state = STATE_DISC_LENGTH;
                 }
-                break;
             case STATE_DISC_LENGTH:
                 cddb_log_debug("...state: DISC LENGTH");
                 if (regexec(REGEX_DISC_LENGTH, line, 2, matches, 0) == 0) {
@@ -1014,7 +995,7 @@ int cddb_query(cddb_conn_t *c, cddb_disc_t *disc)
 
     cddb_log_debug("cddb_query()");
     /* clear previous query result set */
-    cddb_query_clear(c);
+    list_flush(c->query_data);
     
     /* recalculate disc ID to make sure it matches the disc data */
     cddb_disc_calc_discid(disc);
@@ -1084,34 +1065,30 @@ int cddb_query(cddb_conn_t *c, cddb_disc_t *disc)
         case 211:                   /* found inexact matches, list follows */
             cddb_log_debug("...(in)exact matches");
             {
-                int query_max = 0;
+                cddb_disc_t *aux;
+
                 while ((line = cddb_read_line(c)) != NULL) {
                     /* end of list? */
                     if (*line == CHR_DOT) {
                         break;
                     }
-                    /* check whether there is enough space in query result set */
-                    if (c->query_cnt >= query_max) {
-                        /* realloc */
-                        query_max += QUERY_RESULT_SET_INC;
-                        c->query_data = realloc(c->query_data, query_max*sizeof(cddb_disc_t*));
-                    }
                     /* clone disc and fill in the blanks */
-                    c->query_data[c->query_cnt] = cddb_disc_clone(disc);
-                    if (!cddb_parse_query_data(c, c->query_data[c->query_cnt], line)) {
+                    aux = cddb_disc_clone(disc);
+                    if (!cddb_parse_query_data(c, aux, line)) {
+                        cddb_disc_destroy(aux);
                         return -1;
                     }
-                    c->query_cnt++;
+                    list_append(c->query_data, aux);
                 }
-                if (c->query_cnt == 0) {
+                if (list_size(c->query_data) == 0) {
                     /* empty result set */
                     cddb_errno_log_error(c, CDDB_ERR_INVALID_RESPONSE);
                     return -1;
                 }
                 /* return first disc in result set */
-                cddb_disc_copy(disc, c->query_data[c->query_idx++]);
+                cddb_disc_copy(disc, (cddb_disc_t *)element_data(list_first(c->query_data)));
             }
-            count = c->query_cnt;
+            count = list_size(c->query_data);
             break;
         case 202:                   /* no match found */
             cddb_log_debug("...no match");
@@ -1142,19 +1119,174 @@ int cddb_query(cddb_conn_t *c, cddb_disc_t *disc)
 
 int cddb_query_next(cddb_conn_t *c, cddb_disc_t *disc)
 {
+    elem_t *aux;
+
     cddb_log_debug("cddb_query_next()");
-    if ((c->query_cnt == 0) || (c->query_idx >= c->query_cnt)) {
+    aux = list_next(c->query_data);
+    if (!aux) {
         /* no more discs */
         cddb_errno_set(c, CDDB_ERR_DISC_NOT_FOUND);
         return FALSE;
     }
     /* return next disc in result set */
-    cddb_disc_copy(disc, c->query_data[c->query_idx++]);
+    cddb_disc_copy(disc, (cddb_disc_t *)element_data(aux));
 
     cddb_errno_set(c, CDDB_ERR_OK);
     return TRUE;
 }
 
+static int cddb_parse_search_data(cddb_conn_t *c, cddb_disc_t **disc,
+                                  char *line, regmatch_t *matches)
+{
+    regmatch_t pre_matches[11];
+    char *buf;
+
+    /* HACK: because of greedy matching of POSIX regular expressions
+       we first need to check whether the prefix remainder also
+       contains a valid match. */
+    buf = cddb_regex_get_string(line, matches, 1);
+    if (regexec(REGEX_TEXT_SEARCH, buf, 11, pre_matches, 0) == 0) {
+        cddb_parse_search_data(c, disc, buf, pre_matches);
+    }
+    free(buf);
+    /* clone so that duplicate matches get correct artist and title */
+    if (*disc) {
+        *disc = cddb_disc_clone(*disc);
+    } else {
+        *disc = cddb_disc_new();
+    }
+    if (*disc == NULL) {
+        cddb_errno_log_error(c, CDDB_ERR_OUT_OF_MEMORY);
+        return FALSE;
+    }
+    /* fill in the results in the new disc */
+    buf = cddb_regex_get_string(line, matches, 2);
+    cddb_disc_set_category_str(*disc, buf);
+    free(buf);
+    cddb_disc_set_discid(*disc, cddb_regex_get_hex(line, matches, 3));
+    if (matches[6].rm_so != -1) {
+        buf = cddb_regex_get_string(line, matches, 6);
+        cddb_disc_set_artist(*disc, buf);
+        free(buf);
+        buf = cddb_regex_get_string(line, matches, 7);
+        cddb_disc_set_title(*disc, buf);
+        free(buf);
+    } else if (matches[8].rm_so != -1) {
+        buf = cddb_regex_get_string(line, matches, 8);
+        cddb_disc_set_artist(*disc, buf);
+        cddb_disc_set_title(*disc, buf);
+        free(buf);
+    } else if (matches[10].rm_so != -1) {
+        /* nothing to do, values should be correct because of cloning */
+    }
+    list_append(c->query_data, *disc);
+    return TRUE;
+}
+
+/**
+ * Build the search parameter string.
+ */
+static void cddb_search_param_str(cddb_search_params_t *params,
+                                  char *buf, int len)
+{
+    char *p = buf;
+    int i;
+
+    /* XXX: to buffer overflow checking */
+    strcpy(p, "&allfields="); p += 11;
+    if (params->fields == SEARCH_ALL) {
+        strcpy(p, "YES"); p += 3;
+    } else {
+        strcpy(p, "NO"); p += 2;
+        if (params->fields & SEARCH_ARTIST) {
+            strcpy(p, "&fields=artist"); p += 14;
+        }
+        if (params->fields & SEARCH_TITLE) {
+            strcpy(p, "&fields=title"); p += 13;
+        }
+        if (params->fields & SEARCH_TRACK) {
+            strcpy(p, "&fields=track"); p += 13;
+        }
+        if (params->fields & SEARCH_OTHER) {
+            strcpy(p, "&fields=rest"); p += 12;
+        }
+    }
+    strcpy(p, "&allcats="); p += 9;
+    if (params->cats == SEARCH_ALL) {
+        strcpy(p, "YES"); p += 3;
+    } else {
+        strcpy(p, "NO"); p += 2;
+        for (i = 0; i < CDDB_CAT_INVALID; i++) {
+            if (params->cats & SEARCHCAT(i)) {
+                strcpy(p, "&cats="); p += 6;
+                strcpy(p, CDDB_CATEGORY[i]); p += strlen(CDDB_CATEGORY[i]);
+            }
+        }
+    }
+    strcpy(p, "&grouping=cats"); p += 14;
+}
+
+int cddb_search(cddb_conn_t *c, cddb_disc_t *disc, const char *str)
+{
+    regmatch_t matches[11];
+    char *line;
+    int count;
+    cddb_disc_t *aux = NULL;
+    char paramstr[1024];        /* big enough! */
+
+    /* NOTE: For server access this function uses the special
+             'cddb_search_conn' connection structure. */
+    cddb_log_debug("cddb_search()");
+    /* copy proxy parameters */
+    cddb_clone_proxy(cddb_search_conn, c);
+    /* clear previous query result set */
+    list_flush(c->query_data);
+    
+    if (!cddb_connect(cddb_search_conn)) {
+        /* connection not OK, copy error code */
+        cddb_errno_set(c, cddb_errno(cddb_search_conn));
+        return -1;
+    }
+
+    /* prepare search parameters string */
+    cddb_search_param_str(&c->srch, paramstr, sizeof(paramstr));
+    
+    /* send query command and check response */
+    if (!cddb_send_cmd(cddb_search_conn, CMD_SEARCH, str, paramstr)) {
+        /* sending command failed, copy error code */
+        cddb_errno_set(c, cddb_errno(cddb_search_conn));
+        return -1;
+    }
+
+    /* parse HTML response page */
+    while ((line = cddb_read_line(cddb_search_conn)) != NULL) {
+        if (regexec(REGEX_TEXT_SEARCH, line, 11, matches, 0) == 0) {
+            /* process matching result line */
+            if (!cddb_parse_search_data(c, &aux, line, matches)) {
+                return -1;
+            }
+        }
+    }
+    /* return first disc in result set */
+    count = list_size(c->query_data);
+    if (count  != 0) {
+        cddb_disc_copy(disc, 
+                       (cddb_disc_t *)element_data(list_first(c->query_data)));
+    }
+    /* close connection */
+    cddb_disconnect(cddb_search_conn);
+
+    cddb_log_debug("...number of matches: %d", count);
+    cddb_errno_set(c, CDDB_ERR_OK);
+    return count;
+}
+
+int cddb_search_next(cddb_conn_t *c, cddb_disc_t *disc)
+{
+    cddb_log_debug("cddb_search_next() ->");
+    return cddb_query_next(c, disc);
+}
+
 int cddb_write_data(cddb_conn_t *c, char *buf, int size, cddb_disc_t *disc)
 {
     int i, remaining;
@@ -1365,3 +1497,70 @@ int cddb_write(cddb_conn_t *c, cddb_disc_t *disc)
     cddb_errno_set(c, CDDB_ERR_OK);
     return TRUE;
 }
+
+int cddb_sites(cddb_conn_t *c)
+{
+    char *msg, *line;
+    int code;
+    cddb_site_t *site;
+
+    cddb_log_debug("cddb_sites()");
+    /* clear previous sites result set */
+    list_flush(c->sites_data);
+
+    if (!cddb_connect(c)) {
+        /* connection not OK */
+        return FALSE;
+    }
+
+    /* send sites command and check response */
+    if (!cddb_send_cmd(c, CMD_SITES)) {
+        return FALSE;
+    }
+    switch (code = cddb_get_response_code(c, &msg)) {
+        case  -1:
+            return FALSE;
+        case 210:                   /* OK, site information follows */
+            break;
+        case 401:                   /* no site information */
+            return FALSE;
+        default:
+            cddb_errno_log_error(c, CDDB_ERR_UNKNOWN);
+            return FALSE;
+    }
+
+    while ((line = cddb_read_line(c)) != NULL) {
+        /* end of list? */
+        if (*line == CHR_DOT) {
+            break;
+        }
+        site = cddb_site_new();
+        if (!site) {
+            cddb_errno_log_error(c, CDDB_ERR_OUT_OF_MEMORY);
+            return FALSE;
+        }
+        if (!cddb_site_parse(site, line)) {
+            /* skip parsing errors */
+            cddb_log_warn("unable to parse site: %s", line);
+            cddb_site_destroy(site);
+            continue;
+        }
+        if (!cddb_site_iconv(c->charset->cd_from_freedb, site)) {
+            cddb_errno_log_error(c, CDDB_ERR_ICONV_FAIL);
+            cddb_site_destroy(site);
+            return FALSE;
+        }
+        if (!list_append(c->sites_data, site)) {
+            cddb_errno_log_error(c, CDDB_ERR_OUT_OF_MEMORY);
+            cddb_site_destroy(site);
+            return FALSE;
+        }
+    }
+
+    /* close connection if using HTTP */
+    if (c->is_http_enabled) {
+        cddb_disconnect(c);
+    }
+
+    return TRUE;
+}
