@@ -1,7 +1,7 @@
 /*
-    $Id: cddb_conn.c,v 1.29 2004/10/15 19:43:29 airborne Exp $
+    $Id: cddb_conn.c,v 1.38 2005/08/03 18:28:22 airborne Exp $
 
-    Copyright (C) 2003, 2004 Kris Verbeeck <airborne@advalvas.be>
+    Copyright (C) 2003, 2004, 2005 Kris Verbeeck <airborne@advalvas.be>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -52,7 +52,14 @@
 /**
  * Send handshake to CDDB server.
  */
-int cddb_handshake(cddb_conn_t *c);
+static int cddb_handshake(cddb_conn_t *c);
+
+/**
+ * Reset proxy authentication credentials.
+ */
+static void cddb_set_http_proxy_auth(cddb_conn_t *c,
+                                     const char *username,
+                                     const char *password);
 
 
 /* --- construction / destruction --- */
@@ -63,10 +70,7 @@ cddb_conn_t *cddb_new(void)
     cddb_conn_t *c;
     const char *s;
 
-#ifdef HAVE_REGEX_H
-    cddb_regex_init();
-#endif
-
+    libcddb_init();             /* initialize globals if not yet done */
     c = (cddb_conn_t*)malloc(sizeof(cddb_conn_t));
     if (c) {
         c->buf_size = DEFAULT_BUF_SIZE;
@@ -89,6 +93,9 @@ cddb_conn_t *cddb_new(void)
         c->is_http_proxy_enabled = FALSE;
         c->http_proxy_server = NULL;
         c->http_proxy_server_port = DEFAULT_PROXY_PORT;
+        c->http_proxy_username = NULL;
+        c->http_proxy_password = NULL;
+        c->http_proxy_auth = NULL;
 
         c->use_cache = CACHE_ON;
         /* construct cache dir '$HOME/[DEFAULT_CACHE]' */
@@ -103,13 +110,16 @@ cddb_conn_t *cddb_new(void)
 
         c->errnum = CDDB_ERR_OK;
 
-        c->query_data = NULL;
-        c->query_idx = 0;
-        c->query_cnt = 0;
+        // XXX: handle out-of-memory
+        c->query_data = list_new((elem_destroy_cb*)cddb_disc_destroy);
+        c->sites_data = list_new((elem_destroy_cb*)cddb_site_destroy);
 
         c->charset = malloc(sizeof(struct cddb_iconv_s));
         c->charset->cd_to_freedb = NULL;
         c->charset->cd_from_freedb = NULL;
+
+        c->srch.fields = SEARCH_ARTIST | SEARCH_TITLE;
+        c->srch.cats = SEARCH_ALL;
     } else {
         cddb_log_crit(cddb_error_str(CDDB_ERR_OUT_OF_MEMORY));
     }
@@ -141,10 +151,14 @@ void cddb_destroy(cddb_conn_t *c)
         FREE_NOT_NULL(c->server_name);
         FREE_NOT_NULL(c->http_path_query);
         FREE_NOT_NULL(c->http_path_submit);
+        FREE_NOT_NULL(c->http_proxy_server);
+        FREE_NOT_NULL(c->http_proxy_username);
+        FREE_NOT_NULL(c->http_proxy_password);
         FREE_NOT_NULL(c->cache_dir);
         FREE_NOT_NULL(c->user);
         FREE_NOT_NULL(c->hostname);
-        cddb_query_clear(c);
+        list_destroy(c->query_data);
+        list_destroy(c->sites_data);
         cddb_close_iconv(c);
         FREE_NOT_NULL(c->charset);
         free(c);
@@ -188,29 +202,115 @@ void cddb_set_buf_size(cddb_conn_t *c, unsigned int size)
     c->line = (char*)malloc(c->buf_size);
 }
 
+cddb_error_t cddb_set_site(cddb_conn_t *c, const cddb_site_t *site)
+{
+    cddb_error_t rv;
+    const char *server, *path;
+    unsigned int port;
+    cddb_protocol_t proto;
+
+    ASSERT_NOT_NULL(c);
+    if ((rv = cddb_site_get_address(site, &server, &port)) != CDDB_ERR_OK) {
+        cddb_errno_set(c, rv);
+        return rv;
+    }
+    if ((proto = cddb_site_get_protocol(site)) == PROTO_UNKNOWN) {
+        cddb_errno_set(c, CDDB_ERR_INVALID);
+        return CDDB_ERR_INVALID;
+    }
+    if ((rv = cddb_site_get_query_path(site, &path)) != CDDB_ERR_OK) {
+        cddb_errno_set(c, rv);
+        return rv;
+    }
+    cddb_set_server_name(c, server);
+    cddb_set_server_port(c, port);
+    if (proto == PROTO_CDDBP) {
+        cddb_http_disable(c);
+    } else {
+        cddb_http_enable(c);
+        cddb_set_http_path_query(c, path);
+    }
+    return cddb_errno_set(c, CDDB_ERR_OK);
+}
+
+const char *cddb_get_server_name(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->server_name;
+    }
+    return NULL;
+}
+
 void cddb_set_server_name(cddb_conn_t *c, const char *server)
 {
     FREE_NOT_NULL(c->server_name);
     c->server_name = strdup(server);
 }
 
+unsigned int cddb_get_server_port(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->server_port;
+    }
+    return 0;
+}
+
 void cddb_set_server_port(cddb_conn_t *c, int port)
 {
     c->server_port = port;
 }
 
+unsigned int cddb_get_timeout(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->timeout;
+    }
+    return 0;
+}
+
+void cddb_set_timeout(cddb_conn_t *c, unsigned int t)
+{
+    if (c) {
+        c->timeout = t;
+    }
+}
+
+const char *cddb_get_http_path_query(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->http_path_query;
+    }
+    return NULL;
+}
+
 void cddb_set_http_path_query(cddb_conn_t *c, const char *path)
 {
     FREE_NOT_NULL(c->http_path_query);
     c->http_path_query = strdup(path);
 }
 
+const char *cddb_get_http_path_submit(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->http_path_submit;
+    }
+    return NULL;
+}
+
 void cddb_set_http_path_submit(cddb_conn_t *c, const char *path)
 {
     FREE_NOT_NULL(c->http_path_submit);
     c->http_path_submit = strdup(path);
 }
 
+unsigned int cddb_is_http_enabled(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->is_http_enabled;
+    }
+    return FALSE;
+}
+
 void cddb_http_enable(cddb_conn_t *c)
 {
     c->is_http_enabled = TRUE;
@@ -223,6 +323,14 @@ void cddb_http_disable(cddb_conn_t *c)
     cddb_errno_set(c, CDDB_ERR_OK);
 }
 
+unsigned int cddb_is_http_proxy_enabled(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->is_http_proxy_enabled;
+    }
+    return FALSE;
+}
+
 void cddb_http_proxy_enable(cddb_conn_t *c)
 {
     /* enabling HTTP proxy implies HTTP, but not vice versa */
@@ -237,17 +345,113 @@ void cddb_http_proxy_disable(cddb_conn_t *c)
     cddb_errno_set(c, CDDB_ERR_OK);
 }
 
+const char *cddb_get_http_proxy_server_name(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->http_proxy_server;
+    }
+    return NULL;
+}
+
 void cddb_set_http_proxy_server_name(cddb_conn_t *c, const char *server)
 {
     FREE_NOT_NULL(c->http_proxy_server);
     c->http_proxy_server = strdup(server);
 }
 
+unsigned int cddb_get_http_proxy_server_port(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->http_proxy_server_port;
+    }
+    return 0;
+}
+
 void cddb_set_http_proxy_server_port(cddb_conn_t *c, int port)
 {
     c->http_proxy_server_port = port;
 }
 
+static void cddb_set_http_proxy_auth(cddb_conn_t *c,
+                                     const char *username, const char *password)
+{
+    int len;
+    char *auth, *auth_b64;
+
+    FREE_NOT_NULL(c->http_proxy_auth);
+    len = 0;
+    if (username != NULL) {
+        len += strlen(username);
+    }
+    if (password != NULL) {
+        len += strlen(password);
+    }
+    len +=  2;                               /* colon and 0-byte */;
+    auth = (char*)malloc(len);
+    snprintf(auth, len, "%s:%s",
+                        (username ? username : ""),
+                        (password ? password : ""));
+    auth_b64 = (char*)malloc(len * 2); /* certainly big enough */
+    cddb_b64_encode(auth_b64, auth);
+    c->http_proxy_auth = strdup(auth_b64);
+    free(auth_b64);
+    free(auth);
+}
+
+const char *cddb_get_http_proxy_username(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->http_proxy_username;
+    }
+    return NULL;
+}
+
+void cddb_set_http_proxy_username(cddb_conn_t *c, const char *username)
+{
+    FREE_NOT_NULL(c->http_proxy_username);
+    if (username) {
+        c->http_proxy_username = strdup(username);
+    }
+    /* remake authentication credentials */
+    cddb_set_http_proxy_auth(c, c->http_proxy_username, c->http_proxy_password);
+}
+
+const char *cddb_get_http_proxy_password(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->http_proxy_password;
+    }
+    return NULL;
+}
+
+void cddb_set_http_proxy_password(cddb_conn_t *c, const char *password)
+{
+    FREE_NOT_NULL(c->http_proxy_password);
+    if (password) {
+        c->http_proxy_password = strdup(password);
+    }
+    /* remake authentication credentials */
+    cddb_set_http_proxy_auth(c, c->http_proxy_username, c->http_proxy_password);
+}
+
+void cddb_set_http_proxy_credentials(cddb_conn_t* c,
+                                     const char *username, const char* password)
+{
+    /* remove cleartext credentials */
+    FREE_NOT_NULL(c->http_proxy_username);
+    FREE_NOT_NULL(c->http_proxy_password);
+    /* remake authentication credentials */
+    cddb_set_http_proxy_auth(c, username, password);
+}
+
+cddb_error_t cddb_errno(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->errnum;
+    }
+    return CDDB_ERR_INVALID;
+}
+
 void cddb_set_client(cddb_conn_t *c, const char *cname, const char *cversion)
 {
     if (cname && cversion) {
@@ -287,6 +491,43 @@ int cddb_set_email_address(cddb_conn_t *c, const char *email)
     return TRUE;
 }
 
+cddb_cache_mode_t cddb_cache_mode(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->use_cache;
+    }
+    return CACHE_OFF;
+}
+
+void cddb_cache_enable(cddb_conn_t *c)
+{
+    if (c) {
+        c->use_cache = CACHE_ON;
+    }
+}
+
+void cddb_cache_only(cddb_conn_t *c)
+{
+    if (c) {
+        c->use_cache = CACHE_ONLY;
+    }
+}
+
+void cddb_cache_disable(cddb_conn_t *c)
+{
+    if (c) {
+        c->use_cache = CACHE_OFF;
+    }
+}
+
+const char *cddb_cache_get_dir(const cddb_conn_t *c)
+{
+    if (c) {
+        return c->cache_dir;;
+    }
+    return NULL;
+}
+
 int cddb_cache_set_dir(cddb_conn_t *c, const char *dir)
 {
     char *home;
@@ -308,11 +549,42 @@ int cddb_cache_set_dir(cddb_conn_t *c, const char *dir)
     return TRUE;
 }
 
+const cddb_site_t *cddb_first_site(cddb_conn_t *c)
+{
+    elem_t *e;
+
+    e = list_first(c->sites_data);
+    if (e) {
+        return element_data(e);
+    }
+    return NULL;
+}
+
+const cddb_site_t *cddb_next_site(cddb_conn_t *c)
+{
+    elem_t *e;
+
+    e = list_next(c->sites_data);
+    if (e) {
+        return element_data(e);
+    }
+    return NULL;
+}
+
+void cddb_search_set_fields(cddb_conn_t *c, unsigned int fields)
+{
+    c->srch.fields = fields;
+}
+
+void cddb_search_set_categories(cddb_conn_t *c, unsigned int cats)
+{
+    c->srch.cats = cats;
+}
 
 /* --- connecting / disconnecting --- */
 
 
-int cddb_handshake(cddb_conn_t *c)
+static int cddb_handshake(cddb_conn_t *c)
 {
     char *msg;
     int code;
@@ -425,3 +697,24 @@ void cddb_disconnect(cddb_conn_t *c)
     }
     cddb_errno_set(c, CDDB_ERR_OK);
 }
+
+
+/* --- miscellaneous --- */
+
+
+void cddb_clone_proxy(cddb_conn_t *dst, cddb_conn_t *src)
+{
+    if (cddb_is_http_proxy_enabled(src)) {
+        /* XXX: optimize? */
+        FREE_NOT_NULL(dst->http_proxy_server);
+        if (src->http_proxy_server) {
+            dst->http_proxy_server = strdup(src->http_proxy_server);
+        }
+        dst->http_proxy_server_port = src->http_proxy_server_port;
+        FREE_NOT_NULL(dst->http_proxy_auth);
+        if (src->http_proxy_auth) {
+            dst->http_proxy_auth = strdup(src->http_proxy_auth);
+        }
+        cddb_http_proxy_enable(dst);
+    }
+}
